#version 330 core

in vec4 colorFromFile;
in vec3 Normal;
in vec3 FragPos;

out vec4 FragColor;

uniform vec4 colorFromEntity;

uniform vec3 viewPos;

struct DirLight {
    vec3 direction;

};  
struct PointLight {    
    vec3 position;
   
    float constant;
    float linear;
    float quadratic;  
};  

struct LightColor {
vec4 ambient;
vec4 diffuse;
vec4 specular;
};


struct SpotLight {
    vec3  position;
    vec3  direction;
    float cutOff;
};

uniform DirLight dirLights[8];
uniform PointLight pointLights[8];
uniform LightColor lightColors[8];
uniform SpotLight spotLights[8];

vec4 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir, LightColor color);  
vec4 CalcPointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir, LightColor color);
vec4 CalcSpotLight(SpotLight light, PointLight plight, vec3 normal, vec3 fragPos, vec3 viewDir, LightColor color);
vec4 objectColor;

vec4 CalcSpotLight(SpotLight light, PointLight plight, vec3 normal, vec3 fragPos, vec3 viewDir, LightColor color){

vec3 lightDir = normalize(light.position - fragPos);


float theta = dot(lightDir, normalize(-light.direction));
    
if(theta > light.cutOff) 
{       
    float diff = max(dot(normal, lightDir), 0.0);
    
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
    
    float distance    = length(light.position - fragPos);
    float attenuation = plight.constant + plight.linear * distance + 
  			     plight.quadratic * (distance * distance);
    if( attenuation != 0.0){
    attenuation = 1.0/ attenuation;
             }
  
    
    vec4 ambient  = color.ambient  * objectColor;
    vec4 diffuse  = color.diffuse  * diff * objectColor;
    vec4 specular = color.specular * spec;
    ambient  *= attenuation;
    diffuse  *= attenuation;
    specular *= attenuation;

    return (ambient + diffuse + specular);
}
else{
  return color.ambient * objectColor;
  }

}


vec4 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir, LightColor color)
{

   
    //umgedreht statt eigentlich -light.direction
    vec3 lightDir = normalize(light.direction);
    
    float diff = max(dot(normal, lightDir), 0.0);
    
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
    
    vec4 ambient  = color.ambient  * objectColor;
    vec4 diffuse  = color.diffuse  * diff * objectColor;
    vec4 specular = color.specular * spec;
    return (ambient + diffuse + specular);
}  

vec4 CalcPointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir, LightColor color)
{
    

    vec3 lightDir = normalize(light.position - fragPos);
    
    float diff = max(dot(normal, lightDir), 0.0);
    
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
    
    float distance    = length(light.position - fragPos);
    float attenuation = light.constant + light.linear * distance + 
  			     light.quadratic * (distance * distance);
    if( attenuation != 0.0){
    attenuation = 1.0/ attenuation;
             }
  
    
    vec4 ambient  = color.ambient  * objectColor;
    vec4 diffuse  = color.diffuse  * diff * objectColor;
    vec4 specular = color.specular * spec ;
    ambient  *= attenuation;
    diffuse  *= attenuation;
    specular *= attenuation;

    return (ambient + diffuse + specular);
       
} 

void main(){

    if(colorFromEntity.x != -1){
    objectColor = colorFromEntity;
    }
    else{
    objectColor = colorFromFile;
    }
    
    vec3 norm = normalize(Normal);
    vec3 viewDir = normalize(viewPos - FragPos);

    vec4 result = vec4(0.0, 0.0, 0.0, 0.0);

    for(int i = 0; i < 8; i++){
    result += CalcDirLight(dirLights[i], norm, viewDir, lightColors[i]);
    result += CalcPointLight(pointLights[i], norm, FragPos, viewDir, lightColors[i]); 
    result += CalcSpotLight(spotLights[i], pointLights[i], norm, FragPos, viewDir, lightColors[i]);
    }   
      
    result.a = objectColor.a;
    FragColor = result;

}
